package org.eclipse.wtp.releng.tools.performance;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

public class Main
{
  private static final String DOWNLOAD_URL = "http://download.eclipse.org/webtools/downloads/";

  public static void main(String[] args)
  {
    try
    {
      String ceiling = System.getProperty("label.ceiling");
      Build latestBuild = null;
      URL url = new URL(DOWNLOAD_URL);
      InputStream is = url.openConnection().getInputStream();
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String s = br.readLine();
      while (s != null)
      {
        String start = "href=\"drops/";
        String end = "/";
        int sIndex = s.indexOf(start);
        int eIndex = s.indexOf(end, sIndex + start.length());
        while (sIndex != -1 && eIndex != -1)
        {
          String buildLabel = s.substring(sIndex + start.length(), eIndex);
          if (buildLabel != null)
          {
            String[] buildFragments = buildLabel.split("-");
            if (buildFragments.length > 2 && buildFragments[0].length() == 1 && buildFragments[2].length() == 12)
            {
              if (ceiling == null || ceiling.length() == 0 || Long.parseLong(buildFragments[2]) <= Long.parseLong(ceiling))
              {
                Build build = new Build(buildFragments[0], buildFragments[1], buildFragments[2].substring(0, 8), buildFragments[2].substring(8, 12));
                if (latestBuild == null || !latestBuild.newer(build))
                {
                  latestBuild = build;
                }
              }
            }
          }
          sIndex = s.indexOf(start, eIndex + end.length());
          eIndex = s.indexOf(end, sIndex + start.length());
        }
        s = br.readLine();
      }
      br.close();
      if (latestBuild != null)
      {
        String eclipseURL = null;
        String emfURL = null;
        String gefURL = null;
        String jemURL = null;
        StringBuffer buildURL = new StringBuffer();
        buildURL.append(DOWNLOAD_URL);
        buildURL.append("drops/");
        buildURL.append(latestBuild.getType()).append("-");
        buildURL.append(latestBuild.getId()).append("-");
        buildURL.append(latestBuild.getDate());
        buildURL.append(latestBuild.getTime());
        buildURL.append("/");
        url = new URL(buildURL.toString());
        is = url.openConnection().getInputStream();
        br = new BufferedReader(new InputStreamReader(is));
        s = br.readLine();
        while (s != null)
        {
          int i = -1;
          if (eclipseURL == null && (i = s.indexOf("eclipse-SDK")) != -1)
            eclipseURL = getURL(s, i);
          if (emfURL == null && (i = s.indexOf("emf-sdo-xsd-SDK")) != -1)
            emfURL = getURL(s, i);
          if (gefURL == null && (i = s.indexOf("GEF-SDK")) != -1)
            gefURL = getURL(s, i);
          if (jemURL == null && (i = s.indexOf("JEM-SDK")) != -1)
            jemURL = getURL(s, i);
          s = br.readLine();
        }
        br.close();
        if (eclipseURL != null && emfURL != null && gefURL != null && jemURL != null)
        {
          StringBuffer outputFile = new StringBuffer();
          if (args.length > 0)
          {
            outputFile.append(args[0]);
            if (!args[0].endsWith("\\") && !args[0].endsWith("/"))
            {
              outputFile.append("/");
            }
          }
          outputFile.append("label.properties");
          BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile.toString()));
          write(bw, "buildType", latestBuild.getType());
          write(bw, "buildId", latestBuild.getId());
          write(bw, "date", latestBuild.getDate());
          write(bw, "time", latestBuild.getTime());
          write(bw, "eclipseURL.linux", eclipseURL);
          write(bw, "eclipseFile.linux", getFile(eclipseURL));
          write(bw, "emfURL", emfURL);
          write(bw, "emfFile", getFile(emfURL));
          write(bw, "gefURL", gefURL);
          write(bw, "gefFile", getFile(gefURL));
          write(bw, "jemURL", jemURL);
          write(bw, "jemFile", getFile(jemURL));
          bw.close();
        }
      }
    }
    catch (MalformedURLException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private static void write(BufferedWriter bw, String key, String value) throws IOException
  {
    bw.write(key);
    bw.write("=");
    bw.write(value);
    bw.newLine();
  }

  private static String getURL(String s, int i)
  {
    int begin = s.substring(0, i).lastIndexOf('\"');
    int end = s.indexOf('\"', i);
    if (begin != -1 && end != -1)
      return s.substring(begin + 1, end);
    else
      return null;
  }

  private static String getFile(String s)
  {
    int i = s.lastIndexOf('/');
    if (i != -1)
      return s.substring(i + 1);
    else
      return s;
  }

  private static class Build
  {
    private String type;
    private String id;
    private String date;
    private String time;

    public Build(String type, String id, String date, String time)
    {
      this.type = type;
      this.id = id;
      this.date = date;
      this.time = time;
    }

    public String getDate()
    {
      return date;
    }

    public void setDate(String date)
    {
      this.date = date;
    }

    public String getId()
    {
      return id;
    }

    public void setId(String id)
    {
      this.id = id;
    }

    public String getTime()
    {
      return time;
    }

    public void setTime(String time)
    {
      this.time = time;
    }

    public String getType()
    {
      return type;
    }

    public void setType(String type)
    {
      this.type = type;
    }

    public boolean newer(Build build)
    {
      int thisDate = Integer.parseInt(date);
      int thatDate = Integer.parseInt(build.getDate());
      if (thisDate == thatDate)
      {
        int thisTime = Integer.parseInt(time);
        int thatTime = Integer.parseInt(build.getTime());
        if (thisTime == thatTime)
          return getBuildTypeWeight(type) > getBuildTypeWeight(build.getType());
        else
          return thisTime > thatTime;
      }
      else
        return thisDate > thatDate;
    }

    private static final String BUILD_TYPE_S = "S";
    private static final String BUILD_TYPE_I = "I";
    private static final String BUILD_TYPE_N = "N";
    private static final String BUILD_TYPE_M = "M";

    private int getBuildTypeWeight(String buildType)
    {
      if (BUILD_TYPE_S.equals(buildType))
        return 40;
      else if (BUILD_TYPE_I.equals(buildType))
        return 30;
      else if (BUILD_TYPE_N.equals(buildType))
        return 20;
      else if (BUILD_TYPE_M.equals(buildType))
        return 10;
      else
        return 0;
    }
  }
}